import std/json, std/logging, std/os, std/strutils, std/tables
import cliutils, docopt, zero_functional

type WdiwtltConfig* = object
  dbPath*: string
  libraryPath*: string
  cfgPath*: string
  cfg*: CombinedConfig

const DEFAULT_CFG_CONTENTS = """{
  "dbPath": "./wdiwtlt.db.json",
  "libraryPath": "./wdiwtlt-library"
}"""

proc loadConfig*(args: Table[string, Value] = initTable[string, Value]()): WdiwtltConfig =
  var cfgLocations = newSeq[string]()

  if args["--config"]: cfgLocations.add($args["--config"])

  cfgLocations.add($getEnv("WDIWTLTRC", ".wdiwtltrc"))

  if existsEnv("HOME"):
    cfgLocations.add($getEnv("HOME") / ".wdiwtltrc")
  elif existsEnv("USERPROFILE"):
    cfgLocations.add($getEnv("USERPROFILE") / ".wdiwtltrc")

  var cfgFilename : string = cfgLocations -->
    fold("", if len(it) > 0: it else: a)

  debug "Loading config from '" & cfgFilename & "'"

  if not fileExists(cfgFilename):
    warn "could not find .wdiwtltrc file: " & cfgFilename
    if isEmptyOrWhitespace(cfgFilename):
      cfgFilename = $getEnv("HOME") & "/.pitrc"
    var cfgFile: File
    try:
      cfgFile = open(cfgFilename, fmWrite)
      cfgFile.write(DEFAULT_CFG_CONTENTS)
    except: warn "could not write default .wdiwtlt to " & cfgFilename
    finally: close(cfgFile)

  var cfgJson: JsonNode
  try:
    cfgJson = parseFile(cfgFilename)
    debug "config values: \p" & cfgJson.pretty
  except: raise newException(IOError,
    "unable to read config file: " & cfgFilename &
    "\p" & getCurrentExceptionMsg())

  let cfg = CombinedConfig(docopt: args, json: cfgJson)
  result = WdiwtltConfig(
    dbPath: cfg.getVal("db-path"),
    libraryPath: cfg.getVal("library-path"),
    cfgPath: cfgFilename,
    cfg: cfg)
