import std/[json, jsonutils, options, strutils, times]
import timeutils, uuids

type
  MetaSource* = enum msTagInfo = "tag info", msFileLocation = "file location"

  BaseModel* = object of RootObj
    id*: UUID
    name*: string

  Album* = object of BaseModel
    imageUri*: Option[string]
    trackTotal*: int
    year*: Option[int]

  Artist* = object of BaseModel
    imageUri*: Option[string]

  Bookmark* = object of BaseModel
    playlistId*: UUID
    mediaFileId*: UUID
    playIdx*: int
    playTimeMs*: int
    userCreated*: bool
    createdAt*: DateTime
    lastUsed*: DateTime

  MediaFile* = object of BaseModel
    comment*: Option[string]
    dateAdded*: DateTime
    discNumber*: Option[string]
    fileHash*: string
    filePath*: string
    imageUri*: Option[string]
    lastPlayed*: Option[DateTime]
    metaInfoSource*: MetaSource
    playCount*: int
    presentLocally*: bool
    trackNumber*: Option[int]

  Playlist* = object of BaseModel
    userCreated*: bool
    mediaFileCount*: int
    copiedFromId: Option[UUID]
    createdAt*: DateTime
    lastUsed*: DateTime

  Tag* = object
    name*: string
    description*: Option[string]

proc `$`*(mf: MediaFile): string =
  if mf.trackNumber.isSome: align($mf.trackNumber.get, 2) & " - " & mf.name
  else: mf.name

proc `$`*(a: Album): string =
  result = a.name
  if a.year.isSome: result &= " (" & $a.year & ")"

proc `$`*(m: BaseModel): string = m.name

proc `$`*(t: Tag): string =
  result = t.name
  if t.description.isSome: result &= t.description.get

proc toJsonHook(uuid: UUID): JsonNode = %($uuid)
proc fromJsonHook(u: var UUID, node: JsonNode) =
  u = parseUUID(node.getStr)

proc toJsonHook(dt: DateTime): JsonNode = %(dt.formatIso8601)
proc fromJsonHook(dt: var DateTime, node: JsonNode) =
  dt = parseIso8601(node.getStr)

proc `%`*(a: Album): JsonNode = toJson(a)
proc `%`*(a: Artist): JsonNode = toJson(a)
proc `%`*(b: Bookmark): JsonNode = toJson(b)
proc `%`*(mf: MediaFile): JsonNode = toJson(mf)
proc `%`*(p: Playlist): JsonNode = toJson(p)
proc `%`*(t: Tag): JsonNode = toJson(t)

proc parseAlbum*(n: JsonNode): Album = result.fromJson(n)
proc parseArtist*(n: JsonNode): Artist = result.fromJson(n)
proc parseBookmark*(n: JsonNode): Bookmark = result.fromJson(n)
proc parseMediaFile*(n: JsonNode): MediaFile = result.fromJson(n)
proc parsePlaylist*(n: JsonNode): Playlist = result.fromJson(n)
proc parseTag*(n: JsonNode): Tag = result.fromJson(n)
